/*global define, require*/
/*jslint vars:true, nomen:true*/
define([
    'testsuite/qunit',
    'jquery',
    'de_epages/design/inc/customizebuttonstyle',
    '$ready!'
], function (QUnit, $, customizeButtonStyle) {
    'use strict';
    QUnit.module('de_epages/design/inc/customizebuttonstyle');

    QUnit.asyncTest("de_epages/design/inc/customizebuttonstyle", function () {

        // We will use this to initialize the dialog with some test data.
        var init = function () {

            var defaults = {
                CustomizableBasketButton: '<button name="AddToBasket" type="submit"><span class="ep-sprite ep-sprite-s" style="background-image: url(/WebRoot/StoreTypes/6.17.0/Store/SF/Icon/BusinessSymbols/ico_s_basket.png)"></span>In den Warenkorb</button>',
                BubbleHelpSource: '/WebRoot/StoreTypes/6.17.0/Store/BO/icons/ico_s_bubblehelp.png'
            };

            var wording = {
                title: 'Schaltfläche \"In den Warenkorb\" anpassen',
                restoreDefaults: 'Auf Standard zurücksetzen',
                replaceButtonWithCustomImage: 'Eigenes Bild hochladen',
                restoreDefaultImage: 'Bild entfernen',
                borderColor: 'Rahmenfarbe',
                borderRadius: 'Rahmenradius',
                backgroundGradient: 'Farbverlauf',
                fontColor: 'Schriftfarbe',
                fontSize: 'Schriftgröße',
                infoWarning: 'Diese Funktion ist nicht in allen Browsern verfügbar. Dies kann bspw. bei älteren Browsern zu Abweichungen bei der Darstellung führen.',
                apply: 'Übernehmen',
                cancel: 'Abbrechen',
                width: 'Breite',
                height: 'Höhe'
            };

            var data = {
                ChangeAction: 'JSONSaveCustomizableBasketButton',
                ObjectID: 'SomeObjectID',
                CustomizableBasketButton: {
                    Font: {
                        Size: 11,
                        Color: '#000'
                    },
                    Padding: {
                        Height: 1,
                        Width: 16
                    },
                    Gradient: {
                        Color1: '#F9F9F9',
                        Color2: '#E2E2E2'
                    },
                    Border: {
                        Radius: 3,
                        Color: '#AAA'
                    },
                    Image: {
                        Path: './../../BO/icons/ico_s_bubblehelp.png',
                        Height: 100,
                        Width: 100
                    }
                }
            };

            var dataNames = ['CustomizableBasketButton'];
            var defaultValues = {
                ChangeAction: 'JSONSaveCustomizableBasketButton',
                ObjectID: 'SomeObjectID',
                CustomizableBasketButton: {
                    Font: {
                        Size: 11,
                        Color: '#BBB'
                    },
                    Padding: {
                        Height: 12,
                        Width: 15
                    },
                    Gradient: {
                        Color1: '#F9F9F9',
                        Color2: '#E2E2E2'
                    },
                    Border: {
                        Radius: 0,
                        Color: '#AAA'
                    },
                    Image: {
                        Path: null,
                        Height: null,
                        Width: null
                    }
                }
            };

            customizeButtonStyle.init(defaults, wording, data, dataNames, defaultValues);
        };

        // Check plugin functions.
        $.each(['init', 'buildContent', 'buildDialog', 'openDialog'], function (i, fn) {

            QUnit.equal(
                $.isFunction(customizeButtonStyle[fn]),
                true,
                '$.isFunction(customizeButtonStyle[' + fn + ']):'
            );

        });

        // Init.
        init();

        // Check, if there is a dialog window with the correct title.
        var title = 'Schaltfläche \"In den Warenkorb\" anpassen',
            correct = false,
            dialog,
            dialogButtons;

        $('.ui-dialog-title')
            .each(function () {
                var $this = $(this);

                correct = ($this.text() === title);

                if (correct) {
                    dialog = $this.closest('.epDialog');
                }
            });

        // Check init.
        QUnit.equal(
            correct,
            true,
            'Dialog window with correct title was initialized:'
        );

        // Open dialog, when built.
        require(['$pub!builtCustomizeButtonsDialog'], function () {

            customizeButtonStyle.openDialog();

            // Check open.
            QUnit.equal(
                dialog.is(':visible'),
                true,
                '*builtCustomizeButtonsDialog* was provided, and dialog window was opened:'
            );

            dialogButtons = dialog.find('.ui-dialog-buttonset button');

            // Change some input values.
            // Sliders.
            dialog.find('.ep-uiSlider-wrap')
                .each(function (i) {
                    $(this)
                        .data('uiUiSlider')
                        ._trigger('slide', {}, {
                            value: 10 + i
                        });
                });
            var newColors = ['#E2E2E2', '#F9F9F9', '#000', '#AAA'];

            // Colorpickers.
            dialog.find('button')
                .not('[name="AddToBasket"]')
                .each(function (i) {
                    var $this = $(this);
                    if ($this.data('epUiColorpicker')) {
                        $this.trigger('colorSelect', [newColors[i]]);
                    }
                });

            // Click *restoreDefaultImage*.
            $.event.trigger('click', {}, dialog.find('.Button-Preview ul li')[1]);

            // Click *restoreDefaults*.
            $.event.trigger('click', {}, dialogButtons[0]);

            // Fake ajax.
            $.ajax = function (o) {
                var result = $.Deferred();
                var data = {
                    ChangeAction: o.data.ChangeAction,
                    ObjectID: o.data.ObjectID,
                    ViewAction: o.data.ViewAction,
                    CustomizableBasketButton: $.parseJSON(o.data.CustomizableBasketButton || "null")
                };
                var expected = {
                    ChangeAction: "JSONSaveCustomizableBasketButton",
                    CustomizableBasketButton: $.parseJSON('{"Font":{"Size":11,"Color":"#BBB"},"Padding":{"Height":12,"Width":15},"Gradient":{"Color1":"#F9F9F9","Color2":"#E2E2E2"},"Border":{"Radius":0,"Color":"#AAA"},"Image":{"Path":null,"Height":null,"Width":null}}'),
                    ObjectID: "SomeObjectID",
                    ViewAction: "JSONViewResponse"
                };
                // Check the data.
                QUnit.deepEqual(data, expected, 'After changes some values and then clicking *restoreDefaults* we have: Submitted *data*:');

                result.resolve({
                    ChangeAction: "JSONSaveCustomizableBasketButton",
                    CustomizableBasketButton: $.parseJSON('{"Font":{"Size":11,"Color":"#BBB"},"Padding":{"Height":12,"Width":15},"Gradient":{"Color1":"#F9F9F9","Color2":"#E2E2E2"},"Border":{"Radius":0,"Color":"#AAA"},"Image":{"Path":null,"Height":null,"Width":null}}'),
                    ObjectID: "SomeObjectID",
                    ViewAction: "JSONViewResponse"
                });

                return result;
            };

            // Click *apply*.
            $.event.trigger('click', {}, dialogButtons[2]);
        });

        QUnit.start();
    });

});