/*
 * @copyright        © Copyright 2006-2018, epages GmbH, All Rights Reserved.
 *
 * @module            de_epages/ebay/inc/ebaytools
 */

/*jslint browser: true, regexp: true*/
/*global define*/

define('de_epages/ebay/inc/ebaytools', [
    'jquery',
], function($, tmplMainmenu) {

    'use strict';

    return {
        /**
         * Search ebay product catalog by GTIN.
         *
         * @param {object} [options] Object with properties to execute a valid ebay search.
         *
         * @return {array|boolean} An array containing the raw products from ebay OR "false" if an error occurs during the request.
         *
         * @since 7.26.0
         */
        searchEbayByGtin: function(options) {
            var value = options.value ? options.value : '',
                onAfterSearch = options.onAfterSearch ? options.onAfterSearch : false,
                objectId;

            // if an object id exists
            if (options.objectId) {
                objectId = options.objectId;

                // initiate ajax request to get results from the ebay API
                ep.ajax({
                    type: "post",
                    dataType: "json",
                    data: {
                        'ChangeAction': 'JSONSearchCatalogProductByGtin',
                        'ViewAction': 'JSONViewResponse',
                        'SearchString': value,
                        'ObjectID': objectId
                    }
                })
                .done(function (data) {
                    var products;

                    if (data.success !== undefined) {
                        // get products array from result json string
                        products = (data.success && data.products) ? JSON.parse(data.products) : false;

                        // if a callback exists => execute callback with products array as parameter
                        if (onAfterSearch && typeof onAfterSearch === "function") {
                            onAfterSearch(products);
                        }
                    }
                })
                .fail(function () {
                  // console.log('fail: ', arguments);
                });
            }
        }
    }
});
